package com.yx.face.controller;

import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.ScanCodeRestCode;
import com.yx.face.service.ScanCodeBoxService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author shisl
 * @package com.yx.face.controller
 * @class ScanCodeBoxController
 * @date 2023/5/31 上午9:32
 * @description 扫码盒子相关API  api/v1/app
 * <p>
 * code=xxxx——xxxx 表示服务器处理结果代码，
 * 0000 触发配置工具中配置的成功行为。
 * 网络传输成功行为配置，只在 code=0000 时生效
 * 网络失败行为在 code＞0008 时生效
 * vgdecoderesult=FD666820230525EUZHCC18171873CB&&devicenumber=0&&otherparams=xxx
 */
@Api(tags = "扫码盒子相关API")
@Slf4j
@RestController
@RequestMapping("api/v1/scan/code")
public class ScanCodeBoxController {

    @Autowired
    private ScanCodeBoxService scanCodeBoxService;

    @ApiOperation("创建数据")
    @PostMapping("/data_create")
    public void dataCreate(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String message = null;
        try {
            Map<String, Object> param = getParam(request);
            if (param.isEmpty()) {
                message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
            }

            if (message == null) {
                log.info("---->request:{}", JSONObject.toJSONString(param));
                //校验数据是否存在
                String code = param.get("vgdecoderesult").toString();
                //校验数据是否存在
                Boolean existFlag = scanCodeBoxService.checkQrcodeExist(code);
                if (!existFlag) {
                    //不存在
                    message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
                }

                //校验数据是否重复
                if (message == null) {//数据已重复提交
                    Boolean addFlag = scanCodeBoxService.addScanCodeBoxCode(code);
                    if (!addFlag) {
                        message = "code=" + ScanCodeRestCode.USED.getCode() + "&&desc=" + ScanCodeRestCode.USED.getMsg();
                    }
                }

                if (message == null) {//提交数据
                    Boolean success = scanCodeBoxService.dataCreate(code);
                    if (!success) {
                        message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
                    }
                }
            }

            if (message == null) {//成功
                message = "code=" + ScanCodeRestCode.SUCCESS.getCode() + "&&desc=" + ScanCodeRestCode.SUCCESS.getMsg();
            }
        } catch (Exception e) {
            e.printStackTrace();
            message = "code=" + ScanCodeRestCode.FAIL.getCode() + "&&desc=" + ScanCodeRestCode.FAIL.getMsg();
        }

        // 以流的形式返回
        ServletOutputStream out = null;
        ByteArrayOutputStream baos = null;
        try {
            InputStream inStream = new ByteArrayInputStream(message.getBytes());
            byte[] buffer = new byte[1024];
            int len;
            baos = new ByteArrayOutputStream();
            while ((len = inStream.read(buffer)) != -1) {
                baos.write(buffer, 0, len);
            }
            out = response.getOutputStream();
            out.write(baos.toByteArray());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            Objects.requireNonNull(baos).flush();
            baos.close();
            Objects.requireNonNull(out).flush();
            out.close();
        }
    }

    private Map<String, Object> getParam(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder();
        try (InputStream is = request.getInputStream();
             BufferedInputStream bis = new BufferedInputStream(is)) {
            byte[] buffer = new byte[1024];
            int read;
            while ((read = bis.read(buffer)) != -1) {
                sb.append(new String(buffer, 0, read, StandardCharsets.UTF_8));
            }
            String paramStr = sb.toString();
            if (!StringUtils.hasText(paramStr)) {
                return Collections.EMPTY_MAP;
            }
            String[] split = paramStr.split("&&");
            List<String> params = Arrays.asList(split);
            Map<String, Object> map = new HashMap<>();
            for (String param : params) {
                String[] splits = param.split("=");
                if (splits.length > 1) {
                    String key = param.split("=")[0];
                    String value = param.split("=")[1];
                    map.put(key, value);
                }
            }
            return map;
        } catch (Exception e) {
            return Collections.EMPTY_MAP;
        }
    }

}
