package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.yx.face.boot.restful.RestDTO;
import com.yx.face.boot.restful.ServiceException;
import com.yx.face.boot.uitls.SMSOrIdCardUtils;
import com.yx.face.dao.AlcoholTestRecordDao;
import com.yx.face.dao.FaceDeviceDao;
import com.yx.face.model.dto.AlcoholTestDto;
import com.yx.face.model.entity.AlcoholTestRecord;
import com.yx.face.model.excel.AlcoholTestRecordExcel;
import com.yx.face.model.vo.AdminVO;
import com.yx.face.model.vo.AlcoholTestRecordVo;
import com.yx.face.model.vo.FaceDeviceVO;
import com.yx.face.model.vo.FaceLogVO;
import com.yx.face.service.AdminService;
import com.yx.face.service.AlcoholTestRecordService;
import lombok.AllArgsConstructor;
import org.apache.commons.io.CopyUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor(onConstructor_ = @Autowired)
public class AlcoholTestRecordServiceImpl implements AlcoholTestRecordService {

    private final AlcoholTestRecordDao alcoholTestRecordDao;

    private final AdminService adminService;

    private final FaceDeviceDao faceDeviceDao;

    /**
     * 酒测记录分页
     *
     * @param alcoholTestDto PageInfo<AlcoholTestRecord>
     * @return PageInfo<AlcoholTestRecord>
     */
    @Override
    public PageInfo<AlcoholTestRecordVo> getAlcoholTestRecordList(RestDTO<AlcoholTestDto> alcoholTestDto) {
        AlcoholTestDto queryCondition = alcoholTestDto.getData();
        PageHelper.startPage(alcoholTestDto.getPageNum(), alcoholTestDto.getPageSize());
        List<AlcoholTestRecord> list = alcoholTestRecordDao.getList(queryCondition);
        PageInfo<AlcoholTestRecord> alcoholTestRecordPageInfo = new PageInfo<>(list);

        List<AlcoholTestRecord> records = alcoholTestRecordPageInfo.getList();
        List<Long> adminList = list.stream().filter(item -> item.getAdminId() != null && item.getAdminId().longValue() != 0l).map(item -> Long.valueOf(String.valueOf(item.getAdminId()))).distinct().collect(Collectors.toList());
        Map<Integer, AdminVO> adminVOMap = null;
        if (adminList != null && adminList.size() > 0) {
            List<AdminVO> adminVOList = adminService.getByAdminlist(adminList);
            adminVOMap = adminVOList.stream().collect(Collectors.toMap(AdminVO::getAdminId, AdminVO -> AdminVO));
        }

        List<String> snList = list.stream().filter(item -> item.getSn() != null).map(AlcoholTestRecord::getSn).distinct().collect(Collectors.toList());
        Map<String, FaceDeviceVO> faceDeviceVOMap = null;
        if (snList != null && snList.size() > 0) {
            List<FaceDeviceVO> faceDeviceVOList = faceDeviceDao.getBySnList(snList);
            faceDeviceVOMap = faceDeviceVOList.stream().collect(Collectors.toMap(FaceDeviceVO::getSn, FaceDeviceVO -> FaceDeviceVO));
        }

        List<AlcoholTestRecordVo> vos = new ArrayList<>();
        for (AlcoholTestRecord record : records) {
            AlcoholTestRecordVo vo = BeanUtil.copyProperties(record, AlcoholTestRecordVo.class);

            if (ObjectUtil.isNotEmpty(record.getExtend6())) {
                if (Integer.parseInt(record.getExtend6()) >= 20) {
                    vo.setIsDrank(1);
                } else {
                    vo.setIsDrank(0);
                }
            }

            if (record.getPhone() != null) {
                vo.setPhone(SMSOrIdCardUtils.hidePhone(record.getPhone()));
            }

            if (record.getCardId() != null) {
                vo.setCardid(SMSOrIdCardUtils.hideIdNumber(record.getCardId()));
            }

            if (record.getVerifyType() != null) {
                vo.setVerifyType(Integer.valueOf(record.getVerifyType()));
            }

            if (record.getRightNo() != null) {
                vo.setRightno(Integer.valueOf(record.getRightNo()));
            }

            if (adminVOMap != null) {
                AdminVO adminVO = adminVOMap.get(record.getAdminId());
                if (Objects.nonNull(adminVO)) {
                    vo.setUsername(adminVO.getUsername());
                }
            }

            if (faceDeviceVOMap != null) {
                FaceDeviceVO faceDeviceVO = faceDeviceVOMap.get(record.getSn());
                if (Objects.nonNull(faceDeviceVO)) {
                    vo.setDeviceName(faceDeviceVO.getName());
                    vo.setOutType(faceDeviceVO.getPass());
                }
            }

            vos.add(vo);
        }
        PageInfo<AlcoholTestRecordVo> vosPageInfo = new PageInfo<>();
        BeanUtils.copyProperties(alcoholTestRecordPageInfo, vosPageInfo);
        vosPageInfo.setList(vos);
        return vosPageInfo;
    }

    @Override
    public void excelOut(AlcoholTestDto dto, HttpServletResponse response) {
        RestDTO<AlcoholTestDto> restDTO = new RestDTO<>();
        restDTO.setPageNum(0);
        restDTO.setPageSize(50000);
        restDTO.setData(dto);
        PageInfo<AlcoholTestRecordVo> pageList = this.getAlcoholTestRecordList(restDTO);
        if(pageList == null || pageList.getList() == null || CollUtil.isEmpty(pageList.getList())){
            throw new ServiceException("没有可导出的记录");
        }
        List<AlcoholTestRecordVo> records = pageList.getList();
        List<AlcoholTestRecordExcel> list = records.stream().map(object -> {
            AlcoholTestRecordExcel item = BeanUtil.copyProperties(object, AlcoholTestRecordExcel.class);

            if (object.getIsDrank() != null) {
                Integer isDrank = object.getIsDrank();
                if (isDrank == 0) item.setIsDrank("否");
                else if (isDrank == 1) item.setIsDrank("是");
                else {
                    item.setIsDrank("未知");
                }
            }

            if (object.getOutType() != null) {
                Integer outType = object.getOutType();
                if (outType == 0) item.setOutType("通用");
                else if (outType == 1) item.setOutType("进门");
                else if (outType == 2) item.setOutType("出门");
                else {
                    item.setOutType("未知");
                }
            }

            if (object.getVerifyType() != null) {
                Integer verifyType = object.getVerifyType();
                if (verifyType == 0) item.setVerifyType("人脸");
                else if (verifyType == 1) item.setVerifyType("卡");
                else if (verifyType == 2) item.setVerifyType("身份证");
                else if (verifyType == 3) item.setVerifyType("二维码");
                else if (verifyType == 4) item.setVerifyType("远程");
                else if (verifyType == 6) item.setVerifyType("IC卡+人脸");
                else {
                    item.setVerifyType("未知");
                }
            }

            if (object.getRightno() != null) {
                Integer rightno = object.getRightno();
                if (rightno == 106) item.setRightno("管理员");
                else if (rightno == 105) item.setRightno("操作员");
                else if (rightno == 104) item.setRightno("普通员工");
                else if (rightno == 103) item.setRightno("访客");
                else if (rightno == 102) item.setRightno("禁止");
                else if (rightno == 101) item.setRightno("黑名单");
                else {
                    item.setRightno("未知");
                }
            }
            return item;
        }).collect(Collectors.toList());
        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = null;
        try {
            fileName = URLEncoder.encode("酒测记录", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), AlcoholTestRecordExcel.class).sheet("酒测记录").doWrite(list);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
