package com.yx.face.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.yx.face.boot.component.aop.MyAnnotation;
import com.yx.face.boot.core.SnowflakeIdWorker;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.FileUploadUtil;
import com.yx.face.boot.uitls.RedisUtil;
import com.yx.face.dao.*;
import com.yx.face.model.dto.AlcoholTestDto;
import com.yx.face.model.dto.DownFaceNew;
import com.yx.face.model.dto.FacePassDTO;
import com.yx.face.model.entity.*;
import com.yx.face.model.vo.SystemLongTermDocumentsVO;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.UserControlEarlyWarningListService;
import com.yx.face.service.feign.TBDeviceFaceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncExecutionAspectSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @description: FaceTBServiceImpl <br>
 * @date: 2021/6/3 10:40 <br>
 * @author: PWB <br>
 */
@Slf4j
@Service
public class FaceTBServiceImpl implements FaceTBService {
    @Resource
    private FacePassDao facePassDao;
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private FaceRequestLogDao faceRequestLogDao;
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private FaceLogDao faceLogDao;
    @Resource
    private ShortTermFaceLogDao shortTermFaceLogDao;
    @Resource
    private SystemLongTermDocumentsDao systemLongTermDocumentsDao;
    @Autowired
    private AsyncTriggerRetryImpl asyncTriggerRetry;
    @Resource
    private FaceTaskDao faceTaskDao;
    @Resource
    private AdminDao adminDao;
    @Autowired
    private TBDeviceFaceService tbDeviceFaceService;

    @Autowired
    private UserControlEarlyWarningListService userControlEarlyWarningListService;
    @Autowired
    private RedisUtil redisUtil;

    @Resource
    private AlcoholTestRecordDao alcoholTestRecordDao;

    @Value("${web.address}")
    private String address;
//
//    @Autowired
//    private NettyClient nettyClient;

//    @Autowired
//    private TaskPoolConfig taskPoolConfig;

    @Override
    public Boolean downFace(Long userId, Date startTime, Date endTime, Integer adminId) {
        List<FaceDevice> snList = faceDeviceDao.getSnList(adminId);
        if (CollectionUtils.isEmpty(snList)) {
            return true;
        }
        UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
        Map<String, Object> result;
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
        FaceRequestLog requestLog;
        Date now = new Date();
        FacePass facePass;
        for (FaceDevice device : snList) {
       /*     facePassId = facePassDao.getFacePassIdBySn(device.getSn(), userId);
            if (facePassId != null) {
                continue;
            }*/
            facePass = new FacePass();
            facePass.setUserId(userId);
            facePass.setDeviceSn(device.getSn());
            facePass.setDeviceId(device.getId());
            facePass.setStartTime(startTime);
            facePass.setEndTime(endTime);
            facePass.setStatus(2);
            facePass.setType(3);
            facePass.setTaskId(String.valueOf(SnowflakeIdWorker.getSnowId()));
            facePass.setUpdateTime(now);
            facePass.setCreateTime(now);
            facePassDao.insertSelective(facePass);
            dataItem.put("pin", userId);
            dataItem.put("name", userInfo.getUsername());
            dataItem.put("validPeriodStart", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, facePass.getStartTime()));
            dataItem.put("validPeriodEnd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, facePass.getEndTime()));
            dataItem.put("userType", 0);
            dataItem.put("photo", userInfo.getAvatar());
            dataItem.put("rightno", 104);
            dataItem.put("cardid", userInfo.getIdNumber());
            data.add(dataItem);
            params.put("SN", device.getSn());
            params.put("Data", data);
            requestLog = new FaceRequestLog();
            requestLog.setName("下发人脸");
            requestLog.setContext("/downUser");
            requestLog.setParams("");
            requestLog.setStatus(0);
            requestLog.setUsedTime(0L);
            requestLog.setUpdateTime(now);
            requestLog.setCreateTime(now);
            faceRequestLogDao.insertSelective(requestLog);
            params.put("taskId", requestLog.getId());
            result = tbDeviceFaceService.downUser(params);
            log.info("下发人脸downFace = {}", result);
        }
        return true;
    }

    @Override
    public List<FacePassDTO> batchDownFace(List<FacePassDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return dtoList;
        }
        List<FaceDevice> snList = faceDeviceDao.getSnList(null);
        if (CollectionUtils.isEmpty(snList)) {
            return dtoList;
        }
        Map<String, Object> result;
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(dtoList.size());
        Map<String, Object> dataItem;
        FaceRequestLog requestLog;
        Date now = new Date();
        for (FaceDevice device : snList) {
            for (FacePassDTO passDTO : dtoList) {
                dataItem = Maps.newHashMapWithExpectedSize(7);
                dataItem.put("pin", passDTO.getUserId());
                dataItem.put("name", passDTO.getUsername());
                dataItem.put("validPeriodStart", passDTO.getStartTime());
                dataItem.put("validPeriodEnd", passDTO.getEndTime());
                dataItem.put("userType", 0);
                dataItem.put("photo", "");
                dataItem.put("rightno", 104);
                data.add(dataItem);
            }
            params.put("SN", device.getSn());
            params.put("Data", data);
            requestLog = new FaceRequestLog();
            requestLog.setName("下发人脸");
            requestLog.setContext("/downUser");
            requestLog.setParams("");
            requestLog.setStatus(0);
            requestLog.setUsedTime(0L);
            requestLog.setUpdateTime(now);
            requestLog.setCreateTime(now);
            faceRequestLogDao.insertSelective(requestLog);
            params.put("taskId", requestLog.getId());
            result = tbDeviceFaceService.downUser(params);
            log.info("下发人脸downFace = {}", result);
        }
        return dtoList;
    }


    @Override
    public Boolean delFace(List<Long> userIds, Integer adminId) {
        List<FaceDevice> snList = faceDeviceDao.getSnList(adminId);
        if (CollectionUtils.isEmpty(snList)) {
            return true;
        }
        boolean userIdsNotEmpty = CollectionUtils.isEmpty(userIds);
        Map<String, Object> result;
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(userIds.size());
        Map<String, Object> dataItem;
        FaceRequestLog requestLog;
        Date now = new Date();
        for (FaceDevice device : snList) {
            requestLog = new FaceRequestLog();
            if (userIdsNotEmpty) {
                for (Long userId : userIds) {
                    dataItem = Maps.newHashMapWithExpectedSize(1);
                    dataItem.put("pin", userId);
                    data.add(dataItem);
                }
                params.put("Data", data);
            } else {
                params.put("Data", "all");
            }
            params.put("SN", device.getSn());
            requestLog.setName("删除人员人脸信息");
            requestLog.setContext("/delUser");
            requestLog.setParams("");
            requestLog.setStatus(0);
            requestLog.setUsedTime(0L);
            requestLog.setUpdateTime(now);
            requestLog.setCreateTime(now);
            faceRequestLogDao.insertSelective(requestLog);
            params.put("taskId", requestLog.getId());
            result = tbDeviceFaceService.delUser(params);
            log.info("删除人员人脸信息downFace = {}", result);
        }
        return true;
    }

    @Override
    public Boolean delFaceBySn(String sn, List<Long> userIds, Integer adminId) {
        Integer deviceId = faceDeviceDao.getIdBySn(sn);
        if (deviceId == null) {
            return true;
        }
        Map<String, Object> result;
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(userIds.size());
        Map<String, Object> dataItem;
        FaceRequestLog requestLog;
        Date now = new Date();
        requestLog = new FaceRequestLog();
        if (CollectionUtils.isEmpty(userIds)) {
            for (Long userId : userIds) {
                dataItem = Maps.newHashMapWithExpectedSize(1);
                dataItem.put("pin", userId);
                data.add(dataItem);
            }
            params.put("Data", data);
        } else {
            params.put("Data", "all");
        }
        params.put("SN", sn);
        requestLog.setName("删除人员人脸信息");
        requestLog.setContext("/delUser");
        requestLog.setParams("");
        requestLog.setStatus(0);
        requestLog.setUsedTime(0L);
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        faceRequestLogDao.insertSelective(requestLog);
        params.put("taskId", requestLog.getId());
        result = tbDeviceFaceService.delUser(params);
        log.info("删除人员人脸信息downFace = {}", result);
        return true;
    }

    /*=============================================【以下为推送】====================================================*/
    @Override
    public Map<String, Object> connect(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        String sn = null;
        String version = null;
        String devIP = null;
        Integer userCount = null;
        Integer swipeCardCount = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (dto.get("version") != null) {
            version = dto.get("version").toString();
        }
        if (dto.get("devIP") != null) {
            devIP = dto.get("devIP").toString();
        }
        if (dto.get("swipeCardCount") != null) {
            swipeCardCount = (Integer) dto.get("swipeCardCount");
        }
        if (dto.get("userCount") != null) {
            userCount = (Integer) dto.get("userCount");
        }
        Integer deviceId = faceDeviceDao.getIdBySn(sn);
        FaceDevice device = new FaceDevice();
        device.setId(deviceId);
        device.setOnline(1);
        device.setPassCount(swipeCardCount);
        device.setUserCount(userCount);
        device.setAppVersion(version);
        device.setInterIp(devIP);
        Date now = new Date();
        device.setUpdateTime(now);
        if (deviceId == null) {
//            device.setSn(sn);
//            device.setEnable(1);
//            device.setCreateTime(now);
//            faceDeviceDao.insertSelective(device);
            return result;
        }
        faceDeviceDao.updateByPrimaryKeySelective(device);
        return result;
    }

    @Override
    public Map<String, Object> disconnect(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (!StringUtils.hasText(sn)) {
            return result;
        }
        Integer deviceId = faceDeviceDao.getIdBySn(sn);
        if (deviceId == null) {
            return result;
        }
        FaceDevice device = new FaceDevice();
        device.setId(deviceId);
        device.setOnline(0);
        device.setUpdateTime(new Date());
        faceDeviceDao.updateByPrimaryKeySelective(device);
        return result;
    }

    @Override
    public Map<String, Object> report(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        String sn = null;
        String version = null;
        String devIP = null;
        Integer userCount = null;
        Integer swipeCardCount = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (!StringUtils.hasText(sn)) {
            return result;
        }
        Map<String, Object> data = (Map<String, Object>) dto.get("data");
        if (CollectionUtils.isEmpty(data)) {
            return result;
        }
        if (data.get("swipeCardCount") != null) {
            swipeCardCount = (Integer) data.get("swipeCardCount");
        }
        if (data.get("userCount") != null) {
            userCount = (Integer) data.get("userCount");
        }
        if (data.get("version") != null) {
            version = data.get("version").toString();
        }

        if (data.get("devIP") != null) {
            devIP = data.get("devIP").toString();
        }
        Integer deviceId = faceDeviceDao.getIdBySn(sn);
        FaceDevice device = new FaceDevice();
        device.setId(deviceId);
        device.setOnline(1);
        device.setPassCount(swipeCardCount);
        device.setUserCount(userCount);
        device.setAppVersion(version);
        device.setInterIp(devIP);
        Date now = new Date();
        device.setUpdateTime(now);
        if (deviceId == null) {
//            device.setSn(sn);
//            device.setEnable(1);
//            device.setCreateTime(now);
//            faceDeviceDao.insertSelective(device);
            return result;
        }
        faceDeviceDao.updateByPrimaryKeySelective(device);
        return result;
    }

    @Override
    @MyAnnotation
    public Map<String, Object> eventRecords(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        //判断是否为空
        if (org.apache.commons.lang3.StringUtils.isBlank(sn)) {
            return result;
        }
        String snId = faceDeviceDao.getFaceDeviceOne(sn);
        //sn是否存在
        if (org.apache.commons.lang3.StringUtils.isBlank(snId)) {
            return result;
        }
        Integer action = null;
        String orgid = "";
        if (dto.get("action") != null) {
            action = (Integer) dto.get("action");
        }
        if (dto.get("orgid") != null) {
            orgid = dto.get("orgid").toString();
        }

        List<Map<String, Object>> dataList = (List) dto.get("data");
        if (CollectionUtils.isEmpty(dataList)) {
            return result;
        }
        Integer recordid;
        String pin;
        String name = "";
        String photo = null;
        String cardid = "";
        String cardIdEx = "";
        Integer verifytype = 0;
        Integer rightno = 104;

        Integer outtype = 0;
        String idCard = "";
        String phone = "";
        //温度
        String extend3 = "";
        String extend1 = "";
        //健康码组装数据
        String extend5 = "";
        //体温是否异常 0 正常 1 异常
        String twStatus = "";
        //健康码状态00 绿码 01 黄码 10 红码
        String jkmStatus = "";
        //健康码状态 0 健康码 1 国康码
        String jkmType = "";
        //阴性 阳性
        String hsStatus = "";
        //疫苗状态 0针 等等依次累加
        String ymStatus = "";
        //行程信息
        String xcInfo = "";

        // * name 姓名
        String sfzName = "";
        //sex 性别
        String sfzSex = "";
        //nation民族
        String sfzNation = "";
        //born出生年月
        String sfzBorn = "";
        //address地址
        String sfzAddress = "";
        //apartment签发机关
        String sfzApartment = "";
        //period有效期限
        String sfzPeriod = "";
        //no身份证号
        String sfzNo = "";
        //country国籍
        String sfzCountry = "";
        //cn_name中文名
        String sfzCnName = "";
        //Idcard_version证件版本
        String sfzIdcardVersion = "";
        //card_type身份证类型
        String sfzCardType = "";
        //photo 身份证照片
        String sfzPhoto = "";

        String verify_score = null;
        FaceLog faceLog;
        Long userId;
        Date logTime;
        long nowL = System.currentTimeMillis();
        Date now = new Date(nowL);
        String faceTime = "";
        for (Map<String, Object> data : dataList) {
            if (data.get("recordid") != null) {
                recordid = (Integer) data.get("recordid");
            } else {
                continue;
            }
            if (data.get("pin") != null) {
                pin = data.get("pin").toString();

            } else {
                continue;
            }
            if (data.get("name") != null) {
                name = data.get("name").toString().replaceAll(" ", "");
            }
            if (data.get("photo") != null) {
                photo = data.get("photo").toString();
            }
            if (data.get("cardid") != null) {
                cardid = data.get("cardid").toString().replaceAll(" ", "");
            }
            if (data.get("cardIdEx") != null) {
                cardIdEx = data.get("cardIdEx").toString();
            }
            if (data.get("verifytype") != null) {
                verifytype = (Integer) data.get("verifytype");
            }
            if (data.get("rightno") != null) {
                rightno = (Integer) data.get("rightno");
            }
            if (data.get("time") != null) {
                faceTime = data.get("time").toString();
            }
            if (data.get("outtype") != null) {
                outtype = (Integer) data.get("outtype");
            }
            if (data.get("phone") != null) {
                phone = data.get("phone").toString().replaceAll(" ", "");
            }
            if (data.get("extend3") != null) {
                extend3 = data.get("extend3").toString();
            }
            if (data.get("extend1") != null) {
                extend1 = data.get("extend1").toString();
            }
            if (data.get("verify_score") != null) {
                verify_score = data.get("verify_score").toString();
            }


            if (data.get("twStatus") != null) {
                twStatus = data.get("twStatus").toString();
            }
            if (data.get("jkmStatus") != null) {
                jkmStatus = data.get("jkmStatus").toString();
            }
            if (data.get("jkmType") != null) {
                jkmType = data.get("jkmType").toString();
            }
            if (data.get("hsStatus") != null) {
                hsStatus = data.get("hsStatus").toString();
                String[] split = hsStatus.split(",");
                hsStatus = split[0];
//                hsStatus = data.get("hsStatus").toString();
//                String[] split = hsStatus.split(",");
//                hsStatus = split[1].trim();

            }
            if (data.get("ymStatus") != null) {
                ymStatus = data.get("ymStatus").toString();
                String[] split = ymStatus.split(";");
                ymStatus = split[0];
//                ymStatus = data.get("ymStatus").toString();
//                String[] split = ymStatus.split(",");
//                ymStatus = split[1];
            }
            if (data.get("xcInfo") != null) {
                xcInfo = data.get("xcInfo").toString();
            }
//            if (verifytype == 2) {

            Map<String, Object> map = BeanUtil.beanToMap(data.get("idCard"));
            if (map != null && !map.isEmpty()) {
                //身份证信息
//                Map<String, Object> map = (Map<String, Object>) data.get("idCard");
                if (map.get("address") != null) {
                    sfzAddress = map.get("address").toString();
                }
                if (map.get("apartment") != null) {
                    sfzApartment = map.get("apartment").toString();
                }
                if (map.get("born") != null) {
                    sfzBorn = map.get("born").toString();
                }
                if (map.get("card_type") != null) {
                    sfzCardType = map.get("card_type").toString();
                }
                if (map.get("cn_name") != null) {
                    sfzCnName = map.get("cn_name").toString();
                }
                if (map.get("country") != null) {
                    sfzCountry = map.get("country").toString();
                }
                if (map.get("idCard_version") != null) {
                    sfzIdcardVersion = map.get("idCard_version").toString();
                }
                if (map.get("name") != null) {
                    sfzName = map.get("name").toString();
                }
                if (map.get("nation") != null) {
                    sfzNation = map.get("nation").toString();
                }
                if (map.get("no") != null) {
                    sfzNo = map.get("no").toString();
                }
                if (map.get("period") != null) {
                    sfzPeriod = map.get("period").toString();
                }
                if (map.get("photo") != null) {
                    sfzPhoto = map.get("photo").toString();
                    if (StringUtils.hasText(sfzPhoto)) {
                        sfzPhoto = buildImage(sfzPhoto, sn + pin);
                    }
                }
                if (map.get("sex") != null) {
                    sfzSex = map.get("sex").toString();
                }
            }

            if (StringUtils.hasText(photo)) {
                photo = buildImage(photo, sn + pin);
            }
            faceLog = new FaceLog();
//            if (verifytype == 0) {
//                userId = Long.valueOf(pin);
//                UserInfo userInfo = userInfoDao.selectByPrimaryKey(userId);
//                if(userInfo != null && userInfo.getPhone() !=null) faceLog.setPhone(userInfo.getPhone());
//            } else {
//                userId = 0L;
//            }

            if (verifytype == 0) {
                try {
                    userId = Long.valueOf(pin);
                } catch (Exception e) {
                    userId = 0L;
                    log.error("---->pin有错误:{}", pin);
                }
            } else {
                userId = 0L;
            }

            faceLog.setRecordid(Long.valueOf(recordid));
            faceLog.setAction(action);
            faceLog.setDeviceSn(sn);
            faceLog.setUserId(userId);
            faceLog.setName(name);
            faceLog.setPhoto(photo);
            faceLog.setCardid(cardid);
            faceLog.setCardidex(cardIdEx);
            faceLog.setVerifyType(verifytype);
            faceLog.setRightno(rightno);
            faceLog.setOutType(outtype);
            faceLog.setFaceTime(faceTime);
            faceLog.setOrgid(orgid);
            faceLog.setPhone(phone);
            faceLog.setExtend3(extend3);
            faceLog.setExtend1(extend1);
            faceLog.setExtend5(extend5);
            faceLog.setTwStatus(twStatus);
            faceLog.setJkmStatus(jkmStatus);
            faceLog.setJkmType(jkmType);
            faceLog.setHsStatus(hsStatus);
            faceLog.setYmStatus(ymStatus);
            faceLog.setXcInfo(xcInfo);
            //身份证模块数据
            faceLog.setSfzSex(sfzSex);
            faceLog.setSfzPhoto(sfzPhoto);
            faceLog.setSfzPeriod(sfzPeriod);
            faceLog.setSfzNo(sfzNo);
            faceLog.setSfzNation(sfzNation);
            faceLog.setSfzName(sfzName);
            faceLog.setSfzIdcardVersion(sfzIdcardVersion);
            faceLog.setSfzCountry(sfzCountry);
            faceLog.setSfzCnName(sfzCnName);
            faceLog.setSfzCardType(sfzCardType);
            faceLog.setSfzBorn(sfzBorn);
            faceLog.setSfzApartment(sfzApartment);
            faceLog.setSfzAddress(sfzAddress);
            FaceDevice faceDevice = faceDeviceDao.getFaceDeviceBySn(sn);
            if (faceDevice != null) {
                faceLog.setAdminId(faceDevice.getAdminId());
            }
            faceLog.setVerifyScore(verify_score);
            faceLog.setCreateTime(now);
            log.info("进出记录推送内容，姓名：{}, 身份证：{}, 设备号：{}", faceLog.getName(), faceLog.getCardid(), faceLog.getDeviceSn());
            if (verifytype == 0 || verifytype == 2) {
                //长期通行记录
                faceLogDao.insertSelective(faceLog);

                //短期通行记录
                shortTermFaceLogDao.insertSelective(BeanUtil.copyProperties(faceLog, ShortTermFaceLog.class));
                if (faceDevice != null) {
                    Admin admin = adminDao.selectByPrimaryKey(faceDevice.getAdminId());
                    if (admin != null) {
                        //预警数据插入
                        userControlEarlyWarningListService.addWarningInformation(admin, faceLog.getCardid(), faceLog.getId());
                    }
                }

                try {
                    //插入酒测记录
                    if (data.get("extend6") != null && !"".equals(data.get("extend6").toString().trim()) && Integer.valueOf(data.get("extend6").toString().trim()).intValue() >= 0) {
                        AlcoholTestDto alcoholTestDto = BeanUtil.copyProperties(faceLog, AlcoholTestDto.class);
                        alcoholTestDto.setExtend6(data.get("extend6").toString());
                        alcoholTestDto.setSn(faceLog.getDeviceSn());
                        alcoholTestDto.setCardid(faceLog.getCardid());
                        alcoholTestDto.setRecordid(faceLog.getRecordid() != null ? String.valueOf(faceLog.getRecordid()) : null);
                        alcoholTestDto.setOuttype(faceLog.getOutType() != null ? String.valueOf(faceLog.getOutType()) : null);
                        alcoholTestDto.setRightno(faceLog.getRightno() != null ? String.valueOf(faceLog.getRightno()) : null);
                        alcoholTestDto.setVerify_score(faceLog.getVerifyScore());
                        alcoholTestDto.setVerifytype(faceLog.getVerifyType() != null ? String.valueOf(faceLog.getVerifyType()) : null);
                        alcoholTestPush(alcoholTestDto);
                    }
                } catch (Exception e) {
                    log.error("--->插入酒测记录失败:{}, {}", faceLog.getDeviceSn(), e);

                }

            }
//            Map<String, String> maps = new HashMap<>(1);
//            maps.put("face-log", JSONObject.toJSONString(faceLog));
//            nettyClient.sendMessage(JSON.toJSONString(maps));

        }
        return result;
    }

    @Override
    public Map<String, Object> pushUsers(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        return result;
    }

    @Override
    public Map<String, Object> delUser(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        if (dto == null) {
            return result;
        }
        String sn = null;
        if (dto.get("SN") != null) {
            sn = dto.get("SN").toString();
        }
        if (!StringUtils.hasText(sn)) {
            return result;
        }
        Object Obj = dto.get("data");
        if (Obj == null) {
            return result;
        }
        List<Map<String, Object>> data = Obj instanceof List ? ((List<Map<String, Object>>) Obj) : null;
        String dataStr;
        if (data == null) {
            dataStr = Obj.toString();
            if ("all".equals(dataStr)) {
                facePassDao.removeFacePassBySn(sn, null);
            }
            return result;
        }
        if (data.isEmpty()) {
            return result;
        }
        for (Map<String, Object> map : data) {
            facePassDao.removeFacePassBySn(sn, (Long) map.get("pin"));
        }
        return result;
    }


    @Override
    public Map<String, Object> pushTaskResult(Map<String, Object> dto) {
        log.info("任务处理结果:{}", JSON.toJSONString(dto));
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        return result;
    }

    @Override
    public Map<String, Object> getTaskId(Map<String, Object> dto) {
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(1);
        result.put("errCode", "00");
        result.put("taskId", SnowflakeIdWorker.getSnowId());
        return result;
    }

    @Override
    public void pushUsersOnline(List<Map<String, String>> list) {
//        for (Map<String, String> map : list) {
//            String taskId = (String) redisUtil.get(map.get("taskId"));
//            if (StringUtils)
//            redisUtil.get()
//            redisUtil.get(map.get("taskId"));
//        }
//
//
//
//
//        //获取员工通道
//        List<FaceDevice> faceDeviceList = faceDeviceDao.selectTypeFaceDev(2);
//        if (faceDeviceList.isEmpty()) {
//            log.info("设备为空===========");
//            return;
//        }
//        //获取员工列表
//        List<SystemLongTermDocumentsVO> systemLongTermDocumentsDaoList = systemLongTermDocumentsDao.getList(42, null);
//        if (faceDeviceList.isEmpty()) {
//            log.info("获取员工列表===========");
//            return;
//        }
//        List<SystemLongTermDocumentsVO> mapList = new ArrayList<>();
//        for (SystemLongTermDocumentsVO vo : systemLongTermDocumentsDaoList) {
//            for (Map<String, String> map : list) {
//                String s = map.get(vo.getIdNumber());
//                if (StringUtils.hasText(s)) {
//                    mapList.add(vo);
//                    break;
//                }
//            }
//        }
//        //取差集
//        List<SystemLongTermDocumentsVO> differenceList = systemLongTermDocumentsDaoList.stream()
//                .filter(item -> !mapList.stream()
//                        .map(e -> e.getIdNumber())
//                        .collect(Collectors.toList())
//                        .contains(item.getIdNumber()))
//                .collect(Collectors.toList());
//        String sn = list.get(0).get("SN");
//        FaceDevice faceDeviceBySn = faceDeviceDao.getFaceDeviceBySn(sn);
//        for (SystemLongTermDocumentsVO vo : differenceList) {
//            faceDeviceList(faceDeviceBySn, vo);
//        }
    }

    @Override
    public Map<String, Object> syncUsers(Map<String, Object> param) {
        log.info("param===========" + param);
        Map map = new HashMap();
        ArrayList<Map<String, Object>> objects = new ArrayList<>();
        Integer taskId = (Integer) param.get("taskId");
        if ((taskId != null && "123456789".equals(taskId.toString())) || ((taskId != null && "987654321".equals(taskId.toString())))) {
            String string = (String) redisUtil.get(taskId.toString());
            List<SystemLongTermDocuments> list = JSON.parseArray(string, SystemLongTermDocuments.class);
            if (list != null && !list.isEmpty()) {
                for (SystemLongTermDocuments documents : list) {
                    Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
                    // 下发参数
                    dataItem.put("pin", documents.getJdyId());
                    dataItem.put("name", documents.getName());
                    dataItem.put("validPeriodStart", DateUtil.format(documents.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
                    dataItem.put("validPeriodEnd", DateUtil.format(documents.getExpireTime(), "yyyy-MM-dd HH:mm:ss"));
                    dataItem.put("userType", 0);
                    dataItem.put("photo", documents.getPhoto());
                    dataItem.put("rightno", 105);
                    dataItem.put("cardid", documents.getIdNumber());
                    objects.add(dataItem);
                }
            }
        }
        String sn = (String) param.get("SN");
        systemLongTermDocumentsDao.updateListSn(sn);
//        redisUtil.delete(taskId.toString());
        map.put("errCode", "00");
        map.put("totalNum", "3");
        map.put("updateTime", "11");
        map.put("data", objects);
        return map;
    }

    @Async(AsyncExecutionAspectSupport.DEFAULT_TASK_EXECUTOR_BEAN_NAME)
    @Override
    public Map<String, Object> downUser(DownFaceNew downFace) {
        Map<String, Object> result = new HashMap<>();
        /*1.先进行数据的校验*/
        String msg = "";//存放错误信息
        if (downFace == null) {
            msg = "请传入参数";
        } else if (downFace.getSn() == null || downFace.getSn().equals("")) {
            msg = "设备编号不能为空";
        } else if (downFace.getUserId() == null) {
            msg = "唯一标识userId不能为空";
        } else if (downFace.getName() == null || downFace.getName().equals("")) {
            msg = "姓名不能为空";
        } else if (downFace.getCardid() == null || downFace.getCardid().equals("")) {
            msg = "身份证不能为空";
        } else if (downFace.getRightno() == null) {
            msg = "权限级别不能为空";
        }
        if (StrUtil.isNotEmpty(msg)) {
            result.put("msg", msg);
            log.error("下发失败，失败原因:{}", msg);
            return result;
        }


        //当前时间
        Date now = new Date();
        //天波人脸服务请求日志表
        FaceRequestLog requestLog;
        //人脸下发表
        FacePass facePass;
        //唯一id
        Long faceTaskId;
        //人脸清除定时任务
        FaceTask faceTask;
        //人脸下发到设备里面
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
        FaceDevice faceDeviceBySn = faceDeviceDao.getFaceDeviceBySn(downFace.getSn());
        if (faceDeviceBySn == null) {
            msg = "设备编号不存在";
            result.put("msg", msg);
            return result;
        }
        /*2.根据角色的不同设置不同的有效时间*/
        Date startTime;
        Date endTime;
        Integer rightno = downFace.getRightno();//角色类型 104：访客 | 105：常客 | 101：黑名单
        //有效开始时间
        if (downFace.getValidPeriodStart() == null) {
            startTime = DateUtils.addMinutes(now, -10);//减10分钟 确保当前时间不会因为波动而没有存进去
        } else {
            startTime = downFace.getValidPeriodStart();
        }
        //有效结束时间
        if (downFace.getValidPeriodEnd() == null) {
            if (rightno == 104) {
                endTime = DateUtils.addYears(now, 3);
            } else if (rightno == 105) {
                endTime = DateUtils.addYears(now, 10);
            } else {
                endTime = DateUtils.addYears(now, 100);
            }
        } else {
            endTime = downFace.getValidPeriodEnd();
        }
        /*3.进行下发*/
        // 下发参数
        dataItem.put("pin", downFace.getUserId());
        dataItem.put("name", downFace.getName());
        dataItem.put("validPeriodStart", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, DateUtils.addMinutes(startTime, -2)));
        dataItem.put("validPeriodEnd", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, endTime));
        dataItem.put("rightno", rightno);
        dataItem.put("userType", rightno == 104 ? 0 : 3);
        if (org.apache.commons.lang3.StringUtils.isNotBlank(downFace.getPhoto())) {
            dataItem.put("photo", downFace.getPhoto());
        }
        dataItem.put("cardid", downFace.getCardid());
        dataItem.put("phone", downFace.getPhone());
        dataItem.put("cardIdEx", downFace.getCardIdEx());
        data.add(dataItem);
        params.put("SN", downFace.getSn());
        params.put("Data", data);

        //人脸下发表
        facePass = new FacePass();
        facePass.setUserId(downFace.getUserId());
        facePass.setDeviceSn(faceDeviceBySn.getSn());
        facePass.setDeviceId(faceDeviceBySn.getId());
        facePass.setStartTime(startTime);
        facePass.setEndTime(endTime);
        facePass.setAdminId(faceDeviceBySn.getAdminId());
        facePass.setStatus(2);
        facePass.setType(rightno == 104 ? 3 : 2);
        facePass.setTaskId(faceDeviceBySn.getAdminId().toString());
        facePass.setUpdateTime(now);
        facePass.setCreateTime(now);
        //添加人脸
        facePassDao.insertSelective(facePass);
        /*4.天波人脸服务请求日志表*/
        requestLog = intoFaceRequestLog("下发人脸", "/downUser");

        params.put("taskId", requestLog.getId());
        /*人脸下发的几种情况*/
        /*1.天波人脸下发*/
        asyncTriggerRetry.send(params, facePass);

        faceTaskId = faceTaskDao.getFaceTaskId(faceDeviceBySn.getSn(), downFace.getUserId());
        faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(facePass.getEndTime());
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(faceDeviceBySn.getSn());
            faceTask.setUserId(downFace.getUserId());
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        } else {
            faceTaskDao.updateByPrimaryKeySelective(faceTask);
        }
        result.put("data", "命令下发成功");
        return result;
    }

    /**
     * 酒测数据推送
     *
     * @param alcoholTestDto AlcoholTestDto
     */
    private void alcoholTestPush(AlcoholTestDto alcoholTestDto) {
        log.info("--->酒测记录推送:{}", JSONObject.toJSONString(alcoholTestDto));
        AlcoholTestRecord alcoholTestRecord = new AlcoholTestRecord();
        BeanUtil.copyProperties(alcoholTestDto, alcoholTestRecord);
        alcoholTestRecord.setCardId(alcoholTestDto.getCardid());
        alcoholTestRecord.setRecordId(alcoholTestDto.getRecordid());
        alcoholTestRecord.setOutType(alcoholTestDto.getOuttype());
        alcoholTestRecord.setRightNo(alcoholTestDto.getRightno());
        alcoholTestRecord.setVerifyScore(alcoholTestDto.getVerify_score());
        alcoholTestRecord.setVerifyType(alcoholTestDto.getVerifytype());
        alcoholTestRecord.setCreateTime(new Date());
        alcoholTestRecordDao.insert(alcoholTestRecord);
    }

    private FaceRequestLog intoFaceRequestLog(String name, String context) {
        FaceRequestLog requestLog = new FaceRequestLog();
        requestLog.setName(name);
        requestLog.setContext(context);
        requestLog.setParams("");
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        Date now = new Date();
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        faceRequestLogDao.insertSelective(requestLog);
        return requestLog;
    }

    @Async
    public void faceDeviceList(FaceDevice faceDevice, SystemLongTermDocumentsVO vo) {
        //天波人脸服务请求日志表
        FaceRequestLog requestLog;
        Date now = new Date();
        //人脸下发表
        FacePass facePass;
        //唯一id
        Long faceTaskId;
        //人脸清除定时任务
        FaceTask faceTask;
        //人脸下发到设备里面
        Map<String, Object> params = Maps.newHashMapWithExpectedSize(3);
        List<Map<String, Object>> data = Lists.newArrayListWithExpectedSize(1);
        Map<String, Object> dataItem = Maps.newHashMapWithExpectedSize(8);
        // 下发参数
        dataItem.put("pin", vo.getCardNo());
        dataItem.put("name", vo.getName());
        dataItem.put("validPeriodStart", DateUtil.format(vo.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
        dataItem.put("validPeriodEnd", DateUtil.format(vo.getExpireTime(), "yyyy-MM-dd HH:mm:ss"));
        dataItem.put("userType", 0);
        dataItem.put("photo", vo.getPhoto());
        dataItem.put("rightno", 105);
        dataItem.put("cardid", vo.getIdNumber());
        data.add(dataItem);
        params.put("SN", faceDevice.getSn());
        params.put("Data", data);
        //天波人脸服务请求日志表
        requestLog = new FaceRequestLog();
        requestLog.setName("下发人脸");
        requestLog.setContext("/downUser");
        requestLog.setParams(JSONObject.toJSONString(params));
        requestLog.setStatus(1);
        requestLog.setUsedTime(0L);
        requestLog.setUpdateTime(now);
        requestLog.setCreateTime(now);
        //人脸下发记录表
        faceRequestLogDao.insertSelective(requestLog);
        params.put("taskId", requestLog.getId());
        //人脸下发表
        facePass = new FacePass();
        facePass.setUserId(Long.valueOf(vo.getCardNo()));
        facePass.setDeviceSn(faceDevice.getSn());
        facePass.setDeviceId(faceDevice.getId());
        facePass.setStartTime(vo.getCreateTime());
        facePass.setEndTime(vo.getExpireTime());
        facePass.setAdminId(vo.getAdminId());
        facePass.setStatus(2);
        facePass.setType(2);
        facePass.setTaskId(vo.getAdminId().toString());
        facePass.setUpdateTime(now);
        facePass.setCreateTime(now);
        //人脸下发
        facePassDao.insertSelective(facePass);
        //数据同步
        asyncTriggerRetry.send(params, facePass);

        faceTaskId = faceTaskDao.getFaceTaskId(faceDevice.getSn(), Long.valueOf(vo.getCardNo()));
        faceTask = new FaceTask();
        faceTask.setId(faceTaskId);
        faceTask.setTaskTime(facePass.getEndTime());
        faceTask.setStatus(0);
        faceTask.setUpdateTime(now);
        if (faceTaskId == null) {
            faceTask.setDeviceSn(faceDevice.getSn());
            faceTask.setUserId(Long.valueOf(vo.getCardNo()));
            faceTask.setCreateTime(now);
            faceTaskDao.insertSelective(faceTask);
        }
        faceTaskDao.updateByPrimaryKeySelective(faceTask);

    }

    private String buildImage(String image, String phone) {
        String path = null;
        if (StringUtils.hasText(image)) {
            long l = System.currentTimeMillis();
            String name = l + UUID.randomUUID().toString().concat(".png");
            path = FileUploadUtil.uploadUtil(image, name);
            return path;
        }
        return path;
    }

}
