package com.yx.face.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yx.face.boot.restful.CommonException;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.OkHttpUtils;
import com.yx.face.dao.ScanCodeBoxCodeDao;
import com.yx.face.model.entity.ScanCodeBoxCode;
import com.yx.face.service.ScanCodeBoxService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * @author shisl
 * @package com.yx.face.service
 * @class ScanCodeBoxServiceImpl
 * @date 2023/5/31 上午9:34
 * @description
 */
@Slf4j
@Service
public class ScanCodeBoxServiceImpl implements ScanCodeBoxService {

    //Bearer

    @Value("${scan.code.box.appId}")
    private String appId;

    @Value("${scan.code.box.queryEntryId}")
    private String queryEntryId;

    @Value("${scan.code.box.hxEntryId}")
    private String hxEntryId;

    @Value("${scan.code.box.appKey}")
    private String appKey;

    @Value("${scan.code.box.batchCreateUrl}")
    private String batchCreateUrl;

    @Value("${scan.code.box.queryListUrl}")
    private String queryListUrl;

    @Resource
    private ScanCodeBoxCodeDao scanCodeBoxCodeDao;

    @Override
    public Boolean dataCreate(String code) {
        try {
            JSONObject obj = new JSONObject();
            obj.put("app_id", appId);
            obj.put("entry_id", hxEntryId);
            obj.put("is_start_workflow", Boolean.TRUE);
            JSONObject dataObj = new JSONObject();
            JSONObject value04 = new JSONObject();
            value04.put("value", code);
            dataObj.put("_widget_1593962871804", value04);
            JSONObject value70 = new JSONObject();
            value70.put("value", DateUtils.parseDateToStr(DateUtils.FORMAT_UTC_ISO8601, DateUtils.addHours(new Date(), -8)));
            dataObj.put("_widget_1593964223170", value70);
            obj.put("data_list", Arrays.asList(dataObj));
            String respStr = OkHttpUtils.builder().addHeader("Authorization", "Bearer " + appKey).url(batchCreateUrl).localPost(obj.toJSONString()).sync();
            log.info("--->扫码盒子创建数据接口响应参数:{}", respStr);
            JSONObject respObj = JSONObject.parseObject(respStr);
            if (respObj.get("status") != null && "success".equals(respObj.get("status").toString())) {
                //创建
                List<String> successIds = JSONArray.parseArray(respObj.get("success_ids").toString(), String.class);
                updateScanCodeBoxCode(code, successIds.get(0));
                return Boolean.TRUE;
            } else {
                delScanCodeBoxCode(code);
                return Boolean.FALSE;
            }
        } catch (Exception e) {
            delScanCodeBoxCode(code);
            log.error("--->扫码盒子创建数据服务异常");
            return Boolean.FALSE;
        }
    }

    private void delScanCodeBoxCode(String code) {
        try {
            List<ScanCodeBoxCode> scanCodeBoxCodes = scanCodeBoxCodeDao.selectByExample(Example.builder(ScanCodeBoxCode.class)
                    .where(WeekendSqls.<ScanCodeBoxCode>custom()
                            .andEqualTo(ScanCodeBoxCode::getCodeNum, code)).build());
            if (scanCodeBoxCodes != null && scanCodeBoxCodes.size() > 0) {
                ScanCodeBoxCode boxCode = scanCodeBoxCodes.get(0);
                scanCodeBoxCodeDao.delete(boxCode);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateScanCodeBoxCode(String code, String successId) {
        try {
            List<ScanCodeBoxCode> scanCodeBoxCodes = scanCodeBoxCodeDao.selectByExample(Example.builder(ScanCodeBoxCode.class)
                    .where(WeekendSqls.<ScanCodeBoxCode>custom()
                            .andEqualTo(ScanCodeBoxCode::getCodeNum, code)).build());
            if (scanCodeBoxCodes != null && scanCodeBoxCodes.size() > 0) {
                ScanCodeBoxCode boxCode = scanCodeBoxCodes.get(0);
                boxCode.setTransactionId(successId);
                scanCodeBoxCodeDao.updateByPrimaryKey(boxCode);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public Boolean addScanCodeBoxCode(String code) {
        try {
            ScanCodeBoxCode bCode = new ScanCodeBoxCode();
            bCode.setCodeNum(code);
            bCode.setCreateTime(new Date());
            scanCodeBoxCodeDao.insert(bCode);
        } catch (Exception e) {
            log.error("--->数据重复插入:{}", code);
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public Boolean checkQrcodeExist(String code) {
        try {
            JSONObject obj = new JSONObject();
            obj.put("app_id", appId);
            obj.put("entry_id", queryEntryId);
            obj.put("limit", 1);
            obj.put("fields", Arrays.asList("_widget_1585702796877"));
            JSONObject filterObj = new JSONObject();
            filterObj.put("rel", "and");
            JSONObject condObj = new JSONObject();
            condObj.put("field", "_widget_1585702796877");
            condObj.put("method", "eq");
            condObj.put("value", Arrays.asList(code));
            filterObj.put("cond", Arrays.asList(condObj));
            obj.put("filter", filterObj);
            String respStr = OkHttpUtils.builder().addHeader("Authorization", "Bearer " + appKey).url(queryListUrl).localPost(obj.toJSONString()).sync();
            log.info("--->扫码盒子查询数据接口响应参数:{}", respStr);
            JSONObject respObj = JSONObject.parseObject(respStr);
            if (respObj.get("data") != null) {
                List<JSONObject> jsonObjects = JSONArray.parseArray(respObj.get("data").toString(), JSONObject.class);
                if (jsonObjects != null && jsonObjects.size() > 0) {
                    return Boolean.TRUE;
                } else {
                    log.info("--->扫码盒子查询数据不存在:{}", code);
                    return Boolean.FALSE;
                }
            } else {
                log.info("--->扫码盒子查询data数据不存在:{}", code);
                return Boolean.FALSE;
            }
        } catch (Exception e) {
            delScanCodeBoxCode(code);
            log.error("--->扫码盒子查询数据服务异常");
            return Boolean.FALSE;
        }

    }

    @Override
    public Boolean checkQrcodeUsed(String code) {
        List<ScanCodeBoxCode> scanCodeBoxCodes = scanCodeBoxCodeDao.selectByExample(Example.builder(ScanCodeBoxCode.class)
                .where(WeekendSqls.<ScanCodeBoxCode>custom()
                        .andEqualTo(ScanCodeBoxCode::getCodeNum, code)).build());
        if (scanCodeBoxCodes == null || scanCodeBoxCodes.size() == 0) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }
}
