package com.yx.face.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Maps;
import com.yx.face.boot.uitls.DateUtils;
import com.yx.face.boot.uitls.FileUploadUtil;
import com.yx.face.dao.*;
import com.yx.face.model.dto.DownFaceNew;
import com.yx.face.model.dto.VisitorRecordsDTO;
import com.yx.face.model.entity.*;
import com.yx.face.service.FaceTBService;
import com.yx.face.service.ThirdFaceService;
import com.yx.face.service.UserInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @description:
 * @ClassName ThirdFaceServiceImpl
 * @Author WXG
 * @Date 2022/6/21 11:04
 */
@Service
@Slf4j
public class ThirdFaceServiceImpl implements ThirdFaceService {
    @Resource
    private FaceDeviceDao faceDeviceDao;
    @Resource
    private UserInfoDao userInfoDao;
    @Resource
    private UserVisitorListDao userVisitorListDao;
    @Resource
    private FaceTBService faceTBService;
    @Resource
    private UserWhitelistDao userWhitelistdao;
    @Resource
    private UserInfoService userInfoService;
    @Resource
    private AdminDao adminDao;

    @Value("${web.address}")
    private String address;
    private Map<String, Object> mapResult(String msg){
        Map<String, Object> result = Maps.newHashMapWithExpectedSize(2);
        result.put("errcode", "0");
        result.put("errmsg", msg);
        return result;
    }
    @Override
    public Map<String, Object> visitorRecords(VisitorRecordsDTO vrd) {
        log.info("开始——第三方访客登记");
        log.info("访客登记信息推送入参,base64:{}", JSON.toJSONString(vrd));

        /*1.基本数据校验*/
        if (!StringUtils.hasText(vrd.getXm())) {return mapResult("姓名不能为空");}
        if (!StringUtils.hasText(vrd.getSfzh())) {return mapResult("身份证号不能为空");}
        if (!StringUtils.hasText(vrd.getScene_xp())) {return mapResult("现场图片不能为空");}
        if (!StringUtils.hasText(vrd.getMachine_id())) {return mapResult("设备号不能为空");}

        if(!vrd.getMachine_id().contains("hh_")){
            vrd.setMachine_id(new StringBuilder().append("hh_").append(vrd.getMachine_id()).toString());
        }

        FaceDevice faceDeviceBySn = faceDeviceDao.getFaceDeviceBySn(vrd.getMachine_id());
        if (faceDeviceBySn == null)return mapResult("该设备没有录入，请添加该设备至对应账号中");
        /*2.照片转换成路径存入*/
        Date now = new Date();
        String base64 = "";
        if(StringUtils.hasText(vrd.getScene_xp())) {
            base64 = vrd.getScene_xp();
            vrd.setScene_xp(buildImage(vrd.getScene_xp()));
        }
        log.info("访客登记信息推送入参:{}", JSON.toJSONString(vrd));
        /*3.userInfo数据填入*/
        Long userId = userInfoService.checkingIdNumberAndPhone(vrd.getSfzh(), vrd.getPhone());
        UserInfo userInfo = null;
        log.info("访客登记UseId:{}", userId);
        if(userId == null){
            userInfo = new UserInfo();
            userInfo.setUsername(vrd.getXm());
            userInfo.setPhone(vrd.getPhone());
            userInfo.setIdNumber(vrd.getSfzh());
            userInfo.setRoleId(1);
            userInfo.setAuthId(0);
            userInfo.setAvatar(vrd.getScene_xp());
            userInfo.setUpdateTime(now);
            userInfo.setCreateTime(now);
            userInfoDao.insertSelective(userInfo);
        }else {
            userInfo = userInfoDao.selectByPrimaryKey(userId);
            userInfo.setUsername(vrd.getXm());
            userInfo.setIdNumber(vrd.getSfzh());
            userInfo.setAvatar(vrd.getScene_xp());
            userInfo.setUpdateTime(now);
            userInfoDao.updateByPrimaryKeySelective(userInfo);
        }
        log.info("访客登记的userInfo数据:{}", userInfo);
        /*4.判断用户是否在常客中
         * 1.如果在常客 更新常客列表
         * 2.如果用户不在常客则以访客进行操作
         * */
        //获取白名单
        UserWhitelist userWhitelist = userWhitelistdao.getByIdNumberAndId(userInfo.getIdNumber(), null, faceDeviceBySn.getAdminId());
        if (userWhitelist == null && StringUtils.hasText(vrd.getPhone())) {
            userWhitelist = userWhitelistdao.getByPhoneAndId(userInfo.getPhone(), null, faceDeviceBySn.getAdminId());
        }

        if(userWhitelist != null){//常客
            /*1.更新常客列表*/
            UserWhitelist uwl = new UserWhitelist();
            uwl.setStatus(1);
            uwl.setUpdateTime(now);
            uwl.setId(userWhitelist.getId());
            uwl.setPhone(userInfo.getPhone());
            uwl.setAvatar(userInfo.getAvatar());
            uwl.setIdNumber(userInfo.getIdNumber());
            userWhitelistdao.updateByPrimaryKeySelective(uwl);
        }else{//访客
            /*2.添加用户至访客中*/
            UserVisitorList userVisitorList = userVisitorListDao.selectByIdNumberAndAdminId(userInfo.getIdNumber(), faceDeviceBySn.getAdminId());
            if (userVisitorList == null && StringUtils.hasText(vrd.getPhone())){
                userVisitorList = userVisitorListDao.getByPhoneAndId(userInfo.getPhone(), faceDeviceBySn.getAdminId());
            }
            UserVisitorList userVisitorOne = new UserVisitorList();
            if (userVisitorList == null) {
                userVisitorOne.setAdminId(faceDeviceBySn.getAdminId());
                userVisitorOne.setCreateTime(now);
                userVisitorOne.setPhone(userInfo.getPhone());
                userVisitorOne.setAvatar(userInfo.getAvatar());
                userVisitorOne.setIdNumber(userInfo.getIdNumber());
                userVisitorOne.setName(userInfo.getUsername());
                userVisitorListDao.insert(userVisitorOne);
            } else {
                userVisitorOne.setAdminId(faceDeviceBySn.getAdminId());
                userVisitorOne.setId(userVisitorList.getId());
                userVisitorOne.setUpdateTime(now);
                userVisitorOne.setAvatar(userInfo.getAvatar());
                userVisitorListDao.updateByPrimaryKeySelective(userVisitorOne);
            }
        }
        /*5.数据下发*/
        List<FaceDevice> snList = faceDeviceDao.getSnList(faceDeviceBySn.getAdminId());
        if (CollectionUtils.isEmpty(snList)) {
            return mapResult("当前账号下无可用设备");
        }
        Date timeStart = DateUtils.addMinutes(now,-5);
        Date timeEnd;
        if(userWhitelist != null){
            timeEnd = DateUtils.addYears(now,10);
        }else {
            timeEnd = DateUtils.addYears(now,3);
        }

        /*根据常访客有效时间来进行下发*/
        for (FaceDevice faceDevice : snList) {
            DownFaceNew downFaceNew = new DownFaceNew();
            downFaceNew.setCardid(vrd.getSfzh());
            downFaceNew.setRightno(userWhitelist == null ? 104 : 105);
            downFaceNew.setPhoto(base64);
            downFaceNew.setName(vrd.getXm());
            downFaceNew.setUserId(userInfo.getId());
            downFaceNew.setSn(faceDevice.getSn());
            downFaceNew.setPhone(vrd.getPhone());
            downFaceNew.setValidPeriodStart(timeStart);
            downFaceNew.setValidPeriodEnd(timeEnd);
            faceTBService.downUser(downFaceNew);
        }
        log.info("完成——第三方访客登记");
        return mapResult("OK");
    }
    private String buildImage(String image) {
        String path = null;
        if (StringUtils.hasText(image)) {
            String name = UUID.randomUUID().toString().concat(".png");
            path = FileUploadUtil.uploadUtil(image, name);
            return path;
        }
        return path;
    }
}
